from django.core import mail
from django.test import TestCase
from django.utils.http import urlsafe_base64_encode
from django.utils.encoding import smart_bytes
from rest_framework_simplejwt.exceptions import TokenError
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework.exceptions import ValidationError, AuthenticationFailed
from authease.auth_core.serializers import *


class UserSerializerTestCase(TestCase):
    def test_user_serializer(self):
        user_data = {
            "email": "test@example.com",
            "password": "password123",
            "confirm_password": "password123",
            "first_name": "John",
            "last_name": "Doe",
        }
        serializer = UserRegisterSerializer(data=user_data)
        self.assertTrue(serializer.is_valid())
        user = serializer.save()

        self.assertEqual(user.email, user_data["email"])
        self.assertEqual(user.first_name, user_data["first_name"])
        
    def test_user_serializer_with_invalid_data(self):
        # Test serializer with missing required fields
        invalid_user_data = {"email": "test@example.com"}
        serializer = UserRegisterSerializer(data=invalid_user_data)
        
        self.assertFalse(serializer.is_valid())
        
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
        
    def test_user_registration_password_mismatch(self):
        invalid_user_data = {
            "email": "test@example.com",
            "password": "password123",
            "confirm_password": "differentpassword",
            "first_name": "John",
            "last_name": "Doe",
        }
        serializer = UserRegisterSerializer(data=invalid_user_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("error", serializer.errors)
        self.assertEqual(
            serializer.errors["error"],
            ["Passwords do not match"]
        )

    def test_user_registration_invalid_email(self):
        invalid_user_data = {
            "email": "invalid-email",
            "password": "password123",
            "confirm_password": "password123",
            "first_name": "John",
            "last_name": "Doe",
        }
        serializer = UserRegisterSerializer(data=invalid_user_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)

    def test_user_registration_duplicate_email(self):
        User.objects.create(
            email="existing@example.com",
            password="newpassword123",
            first_name="New",
            last_name="User"
        )
        duplicate_user_data = {
            "email": "existing@example.com",
            "password": "newpassword123",
            "confirm_password": "newpassword123",
            "first_name": "New",
            "last_name": "User",
        }
        serializer = UserRegisterSerializer(data=duplicate_user_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)
        self.assertEqual(
            serializer.errors["email"],
            ["User with this Email Address already exists."]
        )

    def test_user_registration_empty_password(self):
        invalid_user_data = {
            "email": "test@example.com",
            "password": "",
            "confirm_password": "",
            "first_name": "John",
            "last_name": "Doe",
        }
        serializer = UserRegisterSerializer(data=invalid_user_data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("password", serializer.errors)


class LoginSerializerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            email="test@example.com",
            password="password123",
            first_name="John",
            last_name="Doe",
            is_verified=True
        )

    def test_login_serializer(self):
        login_data = {
            "email": "test@example.com",
            "password": "password123",
        }
        serializer = LoginSerializer(data=login_data)
        self.assertTrue(serializer.is_valid())
        response = serializer.validated_data
        self.assertEqual(response['email'], self.user.email)
        self.assertEqual(response['full_name'], self.user.get_full_name())
        self.assertIn('access_token', response)
        self.assertIn('refresh_token', response)

    def test_login_serializer_invalid_credentials(self):
        data = {
            "email": "test@example.com",
            "password": "wrongpassword",
        }
        serializer = LoginSerializer(data=data, context={'request': None})
        with self.assertRaises(AuthenticationFailed):
            serializer.is_valid(raise_exception=True)

    def test_login_serializer_unverified_user(self):
        unverified_user = User.objects.create_user(
            email="unverified@example.com",
            password="password123",
            first_name="Jane",
            last_name="Doe",
            is_verified=False
        )
        data = {
            "email": unverified_user.email,
            "password": "password123",
        }
        serializer = LoginSerializer(data=data, context={'request': None})
        with self.assertRaises(AuthenticationFailed):
            serializer.is_valid(raise_exception=True)

    def test_login_serializer_missing_email(self):
        data = {
            "password": "password123",
        }
        serializer = LoginSerializer(data=data, context={'request': None})
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)
        self.assertEqual(
            serializer.errors["email"],
            ["This field is required."]
        )

    def test_login_serializer_missing_password(self):
        data = {
            "email": "test@example.com",
        }
        serializer = LoginSerializer(data=data, context={'request': None})
        self.assertFalse(serializer.is_valid())
        self.assertIn("password", serializer.errors)
        self.assertEqual(
            serializer.errors['password'],
            ["This field is required."]
        )


class PasswordResetRequestSerializerTestCase(TestCase):
    def setUp(self):
        # Create a user for testing
        self.user = User.objects.create_user(
            email="test@example.com",
            password="password123",
            first_name="John",
            last_name="Doe",
            is_verified=True
        )

    def test_password_reset_request_success(self):
        data = {
            "email": self.user.email,
        }
        serializer = PasswordResetRequestSerializer(data=data, context={'request': None})
        self.assertTrue(serializer.is_valid())

        # Check if an email was sent
        self.assertEqual(len(mail.outbox), 1)  # One email should have been sent
        email = mail.outbox[0]
        self.assertIn("Reset your Password", email.subject)
        self.assertIn("Hi,", email.body)
        self.assertIn("http://localhost:8000/api/v1/auth/password_reset_confirm/", email.body)  # Replace with the actual URL pattern

    def test_password_reset_request_user_not_found(self):
        data = {
            "email": "nonexistent@example.com",
        }
        serializer = PasswordResetRequestSerializer(data=data, context={'request': None})
        self.assertFalse(serializer.is_valid())  # Should be false because the user does not exist
        # Check if an email was sent
        self.assertEqual(len(mail.outbox), 0)  # No email should have been sent

    def test_password_reset_request_invalid_email(self):
        data = {
            "email": "invalid-email",
        }
        serializer = PasswordResetRequestSerializer(data=data, context={'request': None})
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)

    def test_password_reset_request_missing_email(self):
        data = {}
        serializer = PasswordResetRequestSerializer(data=data, context={'request': None})
        self.assertFalse(serializer.is_valid())
        self.assertIn("email", serializer.errors)


class SetNewPasswordSerializerTestCase(TestCase):
    def setUp(self):
        # Create a test user
        self.user = User.objects.create_user(
            email='test@example.com',
            password='oldpassword123',
            first_name='John',
            last_name='Doe'
        )
        self.user.is_verified = True
        self.user.save()

    def test_password_reset_success(self):
        # Generate a valid token
        uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        token = PasswordResetTokenGenerator().make_token(self.user)
        data = {
            'password': 'newpassword123',
            'confirm_password': 'newpassword123',
            'token': token,
            'uidb64': uidb64
        }
        serializer = SetNewPasswordSerializer(data=data)
        self.assertTrue(serializer.is_valid())
        # Password reset happens during validation
        serializer.validated_data  # This triggers validation and saving of the new password
        
        # Verify the user's password is updated
        self.user.refresh_from_db()
        self.assertTrue(self.user.check_password('newpassword123')) # check for the new password has been updated
        self.assertFalse(self.user.check_password('oldpassword123')) # check for the old password is not still saved

    def test_invalid_token(self):
        uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        invalid_token = 'invalidtoken'
        data = {
            'password': 'newpassword123',
            'confirm_password': 'newpassword123',
            'token': invalid_token,
            'uidb64': uidb64
        }
        serializer = SetNewPasswordSerializer(data=data)
        with self.assertRaises(AuthenticationFailed):
            serializer.is_valid(raise_exception=True)
            # Password reset happens during validation
            serializer.validated_data  # This triggers validation and saving of the new password
            
            # Verify the user's password is updated
            self.user.refresh_from_db()
            self.assertFalse(self.user.check_password('newpassword123')) # check for the new password has been updated
            self.assertTrue(self.user.check_password('oldpassword123')) # check for the old password is not still saved

    def test_invalid_uid(self):
        invalid_uidb64 = 'invaliduid'
        token = PasswordResetTokenGenerator().make_token(self.user)
        data = {
            'password': 'newpassword123',
            'confirm_password': 'newpassword123',
            'token': token,
            'uidb64': invalid_uidb64
        }
        serializer = SetNewPasswordSerializer(data=data)
        with self.assertRaises(AuthenticationFailed):
            serializer.is_valid(raise_exception=True)
            # Password reset happens during validation
            serializer.validated_data  # This triggers validation and saving of the new password
            
            # Verify the user's password is updated
            self.user.refresh_from_db()
            self.assertFalse(self.user.check_password('newpassword123')) # check for the new password has been updated
            self.assertTrue(self.user.check_password('oldpassword123')) # check for the old password is not still saved

    def test_password_mismatch(self):
        uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        token = PasswordResetTokenGenerator().make_token(self.user)
        data = {
            'password': 'newpassword123',
            'confirm_password': 'mismatchpassword123',
            'token': token,
            'uidb64': uidb64
        }
        serializer = SetNewPasswordSerializer(data=data)
        with self.assertRaises(AuthenticationFailed):
            serializer.is_valid(raise_exception=True)
            # Password reset happens during validation
            serializer.validated_data  # This triggers validation and saving of the new password
            
            # Verify the user's password is updated
            self.user.refresh_from_db()
            self.assertFalse(self.user.check_password('newpassword123')) # check for the new password has been updated
            self.assertTrue(self.user.check_password('oldpassword123')) # check for the old password is not still saved

    def test_missing_token_or_uid(self):
        data = {
            'password': 'newpassword123',
            'confirm_password': 'newpassword123'
        }
        serializer = SetNewPasswordSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('token', serializer.errors)
        self.assertIn('uidb64', serializer.errors)

    def test_empty_password_fields(self):
        uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        token = PasswordResetTokenGenerator().make_token(self.user)
        data = {
            'password': '',
            'confirm_password': '',
            'token': token,
            'uidb64': uidb64
        }
        serializer = SetNewPasswordSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn('password', serializer.errors)


class LogoutSerializerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            email="testuser@example.com",
            first_name="John",
            last_name="Doe",
            password="password123"
        )
        self.valid_refresh_token = str(RefreshToken.for_user(self.user))
        self.invalid_refresh_token = "invalidtoken"

    def test_logout_success(self):
        data = {'refresh_token': str(self.valid_refresh_token)}
        serializer = LogoutSerializer(data=data)
        
        self.assertTrue(serializer.is_valid())
        serializer.save()  # This should blacklist the token

        # Ensure the token is blacklisted by catching the TokenError
        with self.assertRaises(TokenError) as context:
            RefreshToken(self.valid_refresh_token).verify()

        # Check the error message
        self.assertEqual(str(context.exception), "Token is blacklisted")
        
    def test_logout_invalid_token(self):
        data = {'refresh_token': self.invalid_refresh_token}
        serializer = LogoutSerializer(data=data)      
        
        self.assertTrue(serializer.is_valid())
        
        with self.assertRaises(ValidationError) as context:
            serializer.save()

        # Check the error message for an invalid token
        self.assertEqual(str(context.exception.detail[0]), "Token is invalid or expired")




from django.core import mail
from django.urls import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient
from authease.auth_core.models import User, OneTimePassword
from django.core import mail
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework_simplejwt.exceptions import TokenError, TokenBackendError

class LoginUserViewTests(TestCase):
    def setUp(self):
        self.client = APIClient()

        self.login_url = reverse('login')  # the login url has been saved as 'login'
        self.user_data = {
            "email": 'testuser@example.com',
            "password": 'testpassword123',
            "first_name": 'Test',
            "last_name": 'User'
        }
        self.user = User.objects.create_user(**self.user_data)


    def test_login_success(self):
        """
        Test successful login with valid credentials.
        """
        # Get the user and set them as active
        self.user = User.objects.get(email=self.user_data["email"])
        self.user.is_verified = True    # a user cannot login without verifying their account
        self.user.save()

        # Attempt to log in with correct credentials
        data = {
            'email': 'testuser@example.com',
            'password': 'testpassword123'
        }
        response = self.client.post(self.login_url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn('access_token', response.data)
        self.assertIn('email', response.data)
        self.assertEqual(response.data['email'], data['email'])
        self.assertIn('full_name', response.data)

    def test_login_invalid_email(self):
        """
        Test login with an invalid email.
        """
        data = {
            'email': 'invalid@example.com',
            'password': 'testpassword123'
        }
        response = self.client.post(self.login_url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(str(response.data['detail']), 'Invalid email or password. Please try again')

    def test_login_invalid_password(self):
        """
        Test login with an invalid password.
        """
        data = {
            'email': 'testuser@example.com',
            'password': 'wrongpassword'
        }
        response = self.client.post(self.login_url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(str(response.data['detail']), 'Invalid email or password. Please try again')

    def test_login_missing_fields(self):
        """
        Test login with missing fields.
        """
        data = {
            'email': '',
            'password': ''
        }
        response = self.client.post(self.login_url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)
        self.assertIn('password', response.data)

    def test_login_with_unverified_email(self):
        """
            Test login with an unverified email.
        """
        data = {
            'email': 'testuser@example.com',
            'password': 'testpassword123'
        }

        # Attempt to log in with correct credentials
        response = self.client.post(self.login_url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(str(response.data['detail']), 'Your account is not verified. Please verify your email address')


class LogoutViewTests(TestCase):
    def setUp(self):
        self.client = APIClient()

        self.logout_url = reverse('logout')  # Ensure you use the correct URL name
        self.user_data = {
            "email": 'testuser@example.com',
            "password": 'testpassword123',
            "first_name": 'Test',
            "last_name": 'User'
        }
        self.user = User.objects.create_user(**self.user_data)

    def test_logout_success(self):
        """
        Test successful logout.
        """
        # Get the user and set them as active
        self.user = User.objects.get(email=self.user_data["email"])
        self.user.is_verified = True    # a user cannot login without verifying their account
        self.user.save()

        # Authenticate the user first by logging in (you can use the login endpoint)
        user_data = {
            "email": "testuser@example.com",
            "password": "testpassword123",
        }

        login_response = self.client.post(reverse("login"), user_data, format='json')

        # Check response status code for successful login (expecting 200 OK)
        self.assertEqual(login_response.status_code, status.HTTP_200_OK)

        # Extract the tokens from the login response
        access_token = login_response.data['access_token']
        refresh_token = login_response.data['refresh_token']

        # Add the access token to the Authorization header for logout request
        self.client.credentials(HTTP_AUTHORIZATION=f'Bearer {access_token}')

        # Make the logout request with the refresh token
        response = self.client.post(self.logout_url, {"refresh_token": refresh_token}, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Test that the refresh token is blacklisted
        with self.assertRaises(TokenError):
            # Try to use the blacklisted refresh token to get a new access token
            blacklisted_token = RefreshToken(refresh_token)
            blacklisted_token.check_blacklist()  # Should raise an error if the token is blacklisted

    def test_logout_without_authentication(self):
        """
        Test logout without authentication.
        """
        # Get the user and set them as active
        self.user = User.objects.get(email=self.user_data["email"])
        self.user.is_verified = True    # a user cannot login without verifying their account
        self.user.save()

        response = self.client.post(self.logout_url, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Authentication credentials were not provided.')

    def test_logout_invalid_token(self):
        """
        Test logout with an invalid token.
        """
        # Get the user and set them as active
        self.user = User.objects.get(email=self.user_data["email"])
        self.user.is_verified = True  # Ensure the user is verified to allow login
        self.user.save()

        # Add the access token to the Authorization header for logout request
        self.client.credentials(HTTP_AUTHORIZATION=f'Bearer {'access_token'}')

        # Make a logout request with an invalid refresh token
        response = self.client.post(self.logout_url, {'refresh_token': 'invalid_token'}, format='json')

        with self.assertRaises(TokenError) as context:
            # Try to use the blacklisted refresh token to get a new access token
            blacklisted_token = RefreshToken('invalid_token')
            blacklisted_token.check_blacklist()  # Should raise an error if the token is blacklisted

        # Check the error message
        self.assertEqual(str(context.exception), "Token is invalid or expired")

        # Check that the response has a 401 status code
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

        # Check that the error message matches the one in the LogoutSerializer's default error message
        self.assertEqual(response.data['messages'][0]['message'], "Token is invalid or expired")


class TestAuthenticationViewTests(TestCase):
    def setUp(self):
        self.client = APIClient()

        self.test_auth_url = reverse('test-auth')  # Ensure you use the correct URL name
        self.user_data = {
            "email": 'testuser@example.com',
            "password": 'testpassword123',
            "first_name": 'Test',
            "last_name": 'User'
        }
        self.user = User.objects.create_user(**self.user_data)

        # Get the user and set them as active
        self.user = User.objects.get(email=self.user_data["email"])
        self.user.is_verified = True    # a user cannot login without verifying their account
        self.user.save()

        self.client.login(email='testuser@example.com', password='testpassword123')  # Log in to get a token
        
    def test_authentication_success(self):
        """
        Test access to an authenticated view.
        """
        # Authenticate the user first by logging in (you can use the login endpoint)
        user_data = {
            "email": "testuser@example.com",
            "password": "testpassword123",
        }

        login_response = self.client.post(reverse("login"), user_data, format='json')

        # Extract the tokens from the login response
        access_token = login_response.data['access_token']

        # Add the access token to the Authorization header for logout request
        self.client.credentials(HTTP_AUTHORIZATION=f'Bearer {access_token}')

        response = self.client.get(self.test_auth_url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['message'], "Hello, authenticated user!")

    def test_authentication_failure(self):
        """
        Test access to an authenticated view without authentication.
        """
        self.client.logout()  # Log out the user
        
        response = self.client.get(self.test_auth_url, format='json')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'Authentication credentials were not provided.')




# Generated by Django 5.1.3 on 2024-11-09 15:01

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_core', '0002_alter_user_options_alter_onetimepassword_code_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='onetimepassword',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



from django.core import mail
from django.urls import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient
from authease.auth_core.models import User, OneTimePassword
from django.core import mail


class RegisterUserViewTests(TestCase):
    def setUp(self):
        self.register_url = reverse('register')  # the register endpoint has been saved as 'register'
        self.valid_user_data = {
            "email": "testuser@example.com",
            "first_name": "Test",
            "last_name": "User",
            "password": "Password123",
            "confirm_password": "Password123"
        }
        self.invalid_user_data = {
            "email": "invalidemail",
            "first_name": "",
            "last_name": "User",
            "password": "short",
            "confirm_password": "differentPassword"
        }

        self.client = APIClient()

    def test_register_successful(self):
        """
            Test registering a user with valid data and sending email.
        """
        response = self.client.post(self.register_url, self.valid_user_data, format='json')
        user = User.objects.get(email=self.valid_user_data["email"])

        # Ensure status is 201 CREATED
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['data']['email'], self.valid_user_data['email'])
        self.assertTrue(user.check_password(self.valid_user_data['password']))

        self.assertEqual(user.is_verified, False)
        self.assertEqual(user.is_staff, False)
        self.assertEqual(user.is_superuser, False)
        self.assertEqual(user.auth_provider, 'email')

        # Check that one email has been sent
        self.assertEqual(len(mail.outbox), 1)

        # Verify email content
        self.assertEqual(mail.outbox[0].to, [self.valid_user_data["email"]])
        self.assertIn("One time passcode for Email verification", mail.outbox[0].subject)
        self.assertIn(user.first_name, mail.outbox[0].body)

        # Clear the outbox after testing
        mail.outbox.clear()
        
        # Check that activation_code is not empty in the database
        code = OneTimePassword.objects.get(user = user)
        self.assertNotEqual(code.code, None)

    def test_register_invalid_email(self):
        """
        Test registering a user with an invalid email format
        """
        invalid_data = self.valid_user_data.copy()
        invalid_data["email"] = "invalidemail"
        
        response = self.client.post(self.register_url, invalid_data, format='json')
        
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)
        self.assertEqual(response.data['email'][0], "Enter a valid email address.")
    
    def test_register_missing_fields(self):
        """
        Test registering a user with missing required fields
        """
        response = self.client.post(self.register_url, {}, format='json')
        
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)
        self.assertIn('first_name', response.data)
        self.assertIn('password', response.data)
    
    def test_register_passwords_do_not_match(self):
        """
        Test registering a user when the password and confirm_password do not match
        """
        invalid_data = self.valid_user_data.copy()
        invalid_data["confirm_password"] = "differentPassword"
        
        response = self.client.post(self.register_url, invalid_data, format='json')
        
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(str(response.data["error"][0]), "Passwords do not match")
    
    def test_register_email_already_exists(self):
        """
        Test registering a user with an email that already exists
        """
        User.objects.create_user(email=self.valid_user_data['email'], password="Password123", first_name=self.valid_user_data['first_name'], last_name=self.valid_user_data['last_name'])
        
        response = self.client.post(self.register_url, self.valid_user_data, format='json')
        
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)
        self.assertEqual(str(response.data['email'][0]), "User with this Email Address already exists.")
    
    def test_register_invalid_method(self):
        """
        Test invalid request method for user registration
        """
        response = self.client.get(self.register_url, format='json')
        
        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)



from django.urls import path
from rest_framework_simplejwt.views import TokenRefreshView
from .views import (
    RegisterUserView, 
    VerifyUserEmail, 
    LoginUserView, 
    TestAuthenticationView, 
    PasswordResetConfirm, 
    PasswordResetRequestView, 
    SetNewPassword, 
    LogoutView,
)


urlpatterns = [
    path('register/', RegisterUserView.as_view(), name='register'),
    path('verify_email/<str:otpcode>', VerifyUserEmail.as_view(), name='verify-email'),
    path('login/', LoginUserView.as_view(), name='login'),
    path('test_auth/', TestAuthenticationView.as_view(), name='test-auth'),
    path('password_reset/', PasswordResetRequestView.as_view(), name='password_reset'),
    path('password_reset_confirm/<uidb64>/<token>/', PasswordResetConfirm.as_view(), name="password-reset-confirm"),
    path('set_new_password/', SetNewPassword.as_view(), name="set-new-password"),
    path('logout/', LogoutView.as_view(), name='logout'),
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),
]



from django.urls import path
from .views import GoogleSignInView, GithubSignInView


urlpatterns = [
    path('google/', GoogleSignInView.as_view(), name='google-sign-in'),
    path('github/', GithubSignInView.as_view(), name='github-sign-in'),
]



from django.conf import settings
from rest_framework import serializers
from .github import Github
from .utils import Google, register_social_user
from rest_framework.exceptions import AuthenticationFailed, ValidationError


class GoogleSignInSerializer(serializers.Serializer):
    access_token = serializers.CharField(min_length=10)

    def validate_access_token(self, access_token):
        google_user_data = Google.validate(access_token)

        try:
            user_id = google_user_data["sub"]
        except:
            raise serializers.ValidationError("This token is invalid or has expired")
        
        if google_user_data['aud'] != settings.GOOGLE_CLIENT_ID:
            raise AuthenticationFailed(detail="Could not verify user")
        
        email = google_user_data['email']
        first_name = google_user_data['given_name']
        last_name = google_user_data['family_name']
        provider = 'google'
        return register_social_user(provider, email, first_name, last_name)



class GithubOauthSerializer(serializers.Serializer):
    code = serializers.CharField(min_length=5)

    def validate_code(self, code):
        access_token = Github.exchange_code_for_token(code)
        if access_token:
            user = Github.retrieve_github_user(access_token)
            print(user)
            full_name = user["name"]
            email = user["email"]
            print(email)
            names = full_name.split(" ")
            first_name = names[1]
            last_name = names[0]
            provider = 'github'
            return register_social_user(provider, email, first_name, last_name)

        else:
            raise ValidationError("Token is invalid or has expired")





import secrets, string
from authease.auth_core.models import User
from django.conf import settings
from django.contrib.auth import authenticate
from google.auth.transport import requests
from google.oauth2 import id_token
from rest_framework.exceptions import AuthenticationFailed


def generate_random_password(length=12):
    characters = string.ascii_letters + string.digits + string.punctuation
    return ''.join(secrets.choice(characters) for _ in range(length))



class Google():
    @staticmethod
    def validate(access_token):
        """
        Validate method Queries the Google oAUTH2 api to fetch the user info
        """
        try:
            id_info = id_token.verify_oauth2_token(access_token, requests.Request(),clock_skew_in_seconds=10)
            if "https://accounts.google.com" in id_info["iss"]:
                return id_info
        except Exception as e:          
            print(f"Token validation error: {e}")
            return "Token is invalid or has expired"


def login_social_user(email, password):
    user = authenticate(email=email, password=password)
    print(email)
    print(password)
    print(user)
    if user is None:
        print("Authentication failed")
        raise AuthenticationFailed('Invalid email or password')
    user_tokens = user.tokens()
    return {
        'email': user.email,
        'full_name': user.get_full_name(),
        'access_token': user_tokens['access'],
        'refresh_token': user_tokens['refresh'] 
    }


def register_social_user(provider, email, first_name, last_name):
    filtered_user_by_email = User.objects.filter(email=email)
    if filtered_user_by_email.exists():
        if provider == filtered_user_by_email[0].auth_provider:
            result = login_social_user(email, filtered_user_by_email[0].password)
            return result
        else:
            raise AuthenticationFailed('You should login with ' + filtered_user_by_email[0].auth_provider)
    else:
        generated_password = generate_random_password()
        new_user = {
            'email': email,
            'first_name': first_name,
            'last_name': last_name,
            'password': generated_password
        }
        registered_user = User.objects.create_user(**new_user)
        registered_user.auth_provider = provider
        registered_user.is_verified = True
        registered_user.save()
        result = login_social_user(registered_user.email, generated_password)
        return result


from django.test import TestCase
from django.core import mail
from authease.auth_core.utils import generateotp, send_code_to_user, send_normal_email
from authease.auth_core.models import OneTimePassword, User


class UtilsTestCase(TestCase):

    def test_generateotp_length(self):
        """Test that OTP is always 6 digits long."""
        otp = generateotp()
        self.assertEqual(len(otp), 6, "OTP should be 6 digits long")

    def test_generateotp_numeric(self):
        """Test that OTP only contains numeric digits."""
        otp = generateotp()
        self.assertTrue(otp.isdigit(), "OTP should contain only numeric digits")

    def test_send_code_to_user(self):
        """Test that OTP is sent to the user and saved in the database."""
        # Create a test user
        user = User.objects.create(email='testuser@example.com', first_name='Test')

        # Call the function to send the OTP
        send_code_to_user(user.email)

        # Check if OTP was created and linked to the correct user
        otp = OneTimePassword.objects.get(user=user)
        self.assertEqual(len(otp.code), 6, "OTP should be 6 digits long")

        # Ensure the email was sent
        self.assertEqual(len(mail.outbox), 1, "One email should have been sent")

        # Check the content of the email
        sent_email = mail.outbox[0]
        self.assertEqual(sent_email.subject, "One time passcode for Email verification")
        self.assertIn(user.first_name, sent_email.body)
        self.assertIn(otp.code, sent_email.body)
        self.assertEqual(sent_email.to, [user.email])

    def test_send_normal_email(self):
        """Test sending a normal email with the correct data."""
        # Sample email data
        email_data = {
            'email_subject': 'Test Subject',
            'email_body': 'This is a test body.',
            'to_email': 'recipient@example.com'
        }

        # Call the function to send a normal email
        send_normal_email(email_data)

        # Ensure the email was sent
        self.assertEqual(len(mail.outbox), 1, "One email should have been sent")

        # Check the content of the email
        sent_email = mail.outbox[0]
        self.assertEqual(sent_email.subject, email_data['email_subject'])
        self.assertEqual(sent_email.body, email_data['email_body'])
        self.assertEqual(sent_email.to, [email_data['to_email']])



# We can use a package that actually generates an otp and has an expiration time 

import random
from django.core.mail import EmailMessage
from .models import User, OneTimePassword
from django.conf import settings
from django.template.loader import render_to_string


def generateotp():
    return ''.join([str(random.randint(0, 9)) for _ in range(6)])



def send_code_to_user(email):
    Subject = "One time passcode for Email verification"
    otp_code = generateotp()
    print(otp_code)
    try:
        user = User.objects.get(email=email)
        site_url = "https://pypi.org/project/authease/"
        site_name = "Auth Ease"
        context = {
            'user_name': user.first_name,
            'site_name': site_name,
            'site_url': site_url,
            'otp_code': otp_code,
            'current_year': '2024',
        }

        # Create HTML content using a template
        email_body = render_to_string('email/verification_email.html', context)

        # email_body = f"Hi {user.first_name}, thanks for signing up on {current_site}. Please verify your email with the \n one time passcode {otp_code}"
        from_name = "Auth Ease"
        from_email = settings.DEFAULT_FROM_EMAIL

        # Set the "From" header with the desired name and email
        from_address = f"{from_name} <{from_email}>"

        send_email = EmailMessage(subject=Subject, body=email_body, from_email=from_address, to=[email])
        send_email.content_subtype = 'html'

        send_email.send(fail_silently=False)

        # Save OTP to the database only if the email is sent successfully
        OneTimePassword.objects.create(user=user, code=otp_code)
    except Exception as e:
        print(f"Error sending email to {email}: {e}")



def send_normal_email(data):
    email = EmailMessage(
        subject=data['email_subject'],
        body=data['email_body'],
        from_email=settings.DEFAULT_FROM_EMAIL,
        to=[data['to_email']]
    )
    email.send(fail_silently=True)





import requests
from django.conf import settings
from rest_framework.exceptions import AuthenticationFailed

class Github():
    
    @staticmethod
    def exchange_code_for_token(code):
        payload = {
            "client_id": settings.GITHUB_CLIENT_ID,
            "client_secret": settings.GITHUB_CLIENT_SECRET,
            "code": code,
        }
        response = requests.post("https://github.com/login/oauth/access_token", params=payload, headers={'Accept': "application/json"})
        result = response.json()
        token = result.get("access_token")
        return token
    
    @staticmethod
    def retrieve_github_user(access_token):
        try:
            headers = {
                "Authorization": f"Bearer {access_token}",
            }
            response = requests.get("https://api.github.com/user", headers=headers)
            user_data = response.json()
            return user_data
        except Exception as e:
            raise AuthenticationFailed(detail="Token is invaid or has expired")



from rest_framework import status
from rest_framework.response import Response
from rest_framework.generics import GenericAPIView
from authease.auth_core.models import OneTimePassword
from authease.auth_core.utils import send_code_to_user
from rest_framework.permissions import IsAuthenticated
from authease.auth_core.serializers import UserRegisterSerializer, LoginSerializer, LogoutSerializer

class RegisterUserView(GenericAPIView):
    serializer_class = UserRegisterSerializer

    def post(self, request):
        user_data = request.data
        serializer = self.serializer_class(data=user_data)

        if serializer.is_valid(raise_exception=True):
            serializer.save()
            user = serializer.data
            send_code_to_user(user["email"])
            return Response(
                {
                    "data": user,
                    "message": f"Hi, {user['first_name']}. Thanks for signing up! A passcode has been sent.",
                },
                status=status.HTTP_201_CREATED,
            )
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class VerifyUserEmail(GenericAPIView):
    def post(self, request, otpcode):

        try:
            user_code_obj = OneTimePassword.objects.get(code=otpcode)

            # Check if the code is expired
            if user_code_obj.is_expired():  # Now using 15 minutes as expiration time
                user_code_obj.delete()  # Delete expired code
                return Response(
                    {"message": "This code has expired. Please request a new one."},
                    status=status.HTTP_400_BAD_REQUEST,
                )
            
            user = user_code_obj.user
            if not user.is_verified:
                user.is_verified = True
                user.save()

                # delete the otp code after being verified
                user_code_obj.delete()
                
                return Response(
                    {"message": "Account email verified successfully"},
                    status=status.HTTP_200_OK,
                )
            return Response(
                {"message": "User is already verified"},
                status=status.HTTP_400_BAD_REQUEST,
            )
        except OneTimePassword.DoesNotExist:
            return Response(
                {"message": "Passcode does not exist"}, status=status.HTTP_404_NOT_FOUND
            )


class LoginUserView(GenericAPIView):
    serializer_class = LoginSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class TestAuthenticationView(GenericAPIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        data = {
            "message": "Hello, authenticated user!"
        }
        return Response(data, status=status.HTTP_200_OK)
    

class LogoutView(GenericAPIView):
    serializer_class = LogoutSerializer
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(status=status.HTTP_200_OK)



from django.db import models
from django.contrib.auth.models import (
    AbstractBaseUser,
    PermissionsMixin,
)
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from .manager import UserManager
from rest_framework_simplejwt.tokens import RefreshToken
from django.core.validators import RegexValidator

# Create your models here.

AUTH_PROVIDERS = {
    'email': 'email',
    'google': 'google',
    'github': 'github',
    'facebook': 'facebook'
}

class User(AbstractBaseUser, PermissionsMixin):
    # Abstractbaseuser has password, last_login, is_active by default

    email = models.EmailField(unique=True, max_length=255, verbose_name= _("Email Address"), help_text=_("Required. Enter a valid email address."))
    first_name = models.CharField(max_length=150, verbose_name=_("First name"))
    last_name = models.CharField(max_length=150, verbose_name=_("Last name"))

    is_staff = models.BooleanField(
        default=False
    )  # must needed, otherwise you won't be able to loginto django-admin.
    is_active = models.BooleanField(
        default=True
    )  # must needed, otherwise you won't be able to loginto django-admin.
    is_superuser = models.BooleanField(
        default=False
    )  # this field inherit from PermissionsMixin.
    is_verified = models.BooleanField(
        default=False
    )
    date_joined = models.DateTimeField(
        auto_now_add=True
    )
    last_login = models.DateTimeField(
        auto_now=True
    )
    auth_provider = models.CharField(max_length=50, default=AUTH_PROVIDERS.get("email"))

    USERNAME_FIELD = "email"

    REQUIRED_FIELDS = ["first_name", "last_name"]

    objects = UserManager()

    def __str__(self):
        return self.email
    
    def get_full_name(self):
        return f"{self.first_name} {self.last_name}"
    
    def tokens(self):
        refresh = RefreshToken.for_user(self)

        return {
            'refresh': str(refresh),
            'access': str(refresh.access_token)
        }
    
    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"


class OneTimePassword(models.Model):
    code_validator = RegexValidator(regex=r'^\d{6}$', message="Code must be 6 digits.")

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    code = models.CharField(max_length=6, unique=True, validators=[code_validator])
    created_at = models.DateTimeField(auto_now_add=True)  # Automatically set timestamp when created

    def is_expired(self, expiration_minutes=15):
        """Check if the code has expired based on a 15-minute expiration time."""
        expiration_time = self.created_at + timezone.timedelta(minutes=expiration_minutes)
        return timezone.now() > expiration_time

    def __str__(self):
        return f"{self.user.first_name} passcode"



from rest_framework import status
from rest_framework.response import Response
from rest_framework.generics import GenericAPIView
from authease.auth_core.models import User
from authease.auth_core.serializers import PasswordResetRequestSerializer, SetNewPasswordSerializer
from django.utils.http import urlsafe_base64_decode
from django.utils.encoding import smart_str, DjangoUnicodeDecodeError
from django.contrib.auth.tokens import PasswordResetTokenGenerator

class PasswordResetRequestView(GenericAPIView):
    serializer_class = PasswordResetRequestSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data, context={"request": request.data})
        serializer.is_valid(raise_exception=True)
        return Response({"message": "Password reset email sent successfully"}, status=status.HTTP_200_OK)


class PasswordResetConfirm(GenericAPIView):
    def get(self, request, uidb64, token):
        try:
            user_id = smart_str(urlsafe_base64_decode(uidb64))
            user = User.objects.get(id=user_id)

            if not PasswordResetTokenGenerator().check_token(user, token):
                return Response({"message": "Invalid token"}, status=status.HTTP_400_BAD_REQUEST)
            
            return Response({
                "success": True,
                "message": "Valid token, please reset your password",
                'uidb64': uidb64,
                'token': token
            }, status=status.HTTP_200_OK)
        
        except User.DoesNotExist:
            return Response({"message": "Invalid user"}, status=status.HTTP_400_BAD_REQUEST)
        
        except DjangoUnicodeDecodeError:
            return Response({"message": "Invalid token"}, status=status.HTTP_400_BAD_REQUEST)


class SetNewPassword(GenericAPIView):
    serializer_class = SetNewPasswordSerializer

    def patch(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        return Response({"message": "Password reset successful"}, status=status.HTTP_200_OK)



import os
from .models import User
from dotenv import load_dotenv
from django.urls import reverse
from .utils import send_normal_email
from rest_framework import serializers
from django.contrib.auth import authenticate
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
from django.utils.encoding import smart_str, smart_bytes, force_str
from rest_framework.exceptions import AuthenticationFailed
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.auth.tokens import PasswordResetTokenGenerator
from rest_framework_simplejwt.tokens import RefreshToken, TokenError

load_dotenv()


class UserRegisterSerializer(serializers.ModelSerializer):
    password=serializers.CharField(min_length=8, max_length=70, write_only=True)
    confirm_password=serializers.CharField(min_length=8, max_length=70, write_only=True)

    class Meta:
        model=User
        fields=["email", "first_name", "last_name", "password", "confirm_password"]

    def validate(self, attrs):
        password = attrs.get("password", "")
        confirm_password = attrs.get("confirm_password", "")

        if password != confirm_password:
            raise serializers.ValidationError("Passwords do not match")
        
        return attrs
    
    def create(self, validated_data):
        user = User.objects.create_user(
            email=validated_data["email"],
            first_name=validated_data["first_name"],
            last_name=validated_data["last_name"],
            password=validated_data["password"],
        )

        return user


class LoginSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(min_length=8, max_length=255)
    password = serializers.CharField(min_length=8, max_length=70, write_only=True)
    full_name = serializers.CharField(max_length=255, read_only=True)
    access_token = serializers.CharField(max_length=255, read_only=True)
    refresh_token = serializers.CharField(max_length=255, read_only=True)

    class Meta:
        model = User
        fields = ["email", "password", "full_name", "access_token", "refresh_token"]

    def validate(self, attrs):
        email = attrs.get("email", "")
        password = attrs.get("password", "")
        request = self.context.get('request')
        user = authenticate(request=request, email=email, password=password)

        if not user:
            raise AuthenticationFailed("Invalid email or password. Please try again")
        
        if not user.is_verified:
            raise AuthenticationFailed("Your account is not verified. Please verify your email address")
        token = user.tokens()

        return {
            'email': user.email,
            'full_name': user.get_full_name(),
            'access_token': token['access'],
            'refresh_token': token['refresh']
        }


class PasswordResetRequestSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(max_length=255)

    class Meta:
        model=User
        fields = ["email"]

    def validate(self, attrs):
        email = attrs.get('email')

        if User.objects.filter(email=email).exists():
            user = User.objects.get(email=email)
            
            # Check if the user is verified
            if not user.is_verified:
                raise serializers.ValidationError("Email is not verified. Please verify your email before resetting the password.")
        
            uidb64 = urlsafe_base64_encode(smart_bytes(user.id))
            token = PasswordResetTokenGenerator().make_token(user)
            request = self.context.get('request')
            relative_link = reverse('password-reset-confirm', kwargs={'uidb64': uidb64, 'token': token})
            absink = f"http://{os.environ.get('DOMAIN')}{relative_link}"

            email_body = f"Hi, Use the link below to reset your password \n {absink}"
            data= {
                'email_body': email_body,
                'email_subject': "Reset your Password",
                'to_email': user.email
            }
            send_normal_email(data)
        else:
            raise serializers.ValidationError("User with this email does not exist")

        return super().validate(attrs)


class SetNewPasswordSerializer(serializers.Serializer):
    password = serializers.CharField(min_length=6, max_length=100, write_only=True)
    confirm_password = serializers.CharField(min_length=6, max_length=100, write_only=True)
    token = serializers.CharField(write_only=True)
    uidb64 = serializers.CharField(write_only=True)

    class Meta:
        fields = [
            "password",
            "confirm_password",
            "token",
            "uidb64"
        ]

    def validate(self, attrs):
        password = attrs.get("password")
        confirm_password = attrs.get("confirm_password")

        if password != confirm_password:
            raise AuthenticationFailed("Password and Confirm Password doesn't match", 401)
        
        try:
            token = attrs.get("token")
            uidb64 = attrs.get("uidb64")
            user_id = force_str(urlsafe_base64_decode(uidb64))
            user = User.objects.get(id=user_id)
            if not PasswordResetTokenGenerator().check_token(user, token):
                raise AuthenticationFailed("The reset link is invalid", 401)
            
            if password != confirm_password:
                raise AuthenticationFailed("Password and Confirm Password doesn't match", 401)
            
            user.set_password(password)
            user.save()
            return user
        except Exception as e:
            raise AuthenticationFailed("The reset link is invalid", 401)


class LogoutSerializer(serializers.Serializer):
    refresh_token = serializers.CharField()

    default_error_messages = {
        "bad_token": "Token is invalid or expired",
    }

    def validate(self, attrs):
        self.token = attrs.get('refresh_token')
        return attrs
    
    def save(self, **kwargs):
        try:
            token = RefreshToken(self.token)
            token.blacklist()

        except TokenError:
            self.fail("bad_token")




from django.test import TestCase
from authease.auth_core.models import User, OneTimePassword


class UserModelTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            email='testuser@example.com',
            first_name='Test',
            last_name='User',
            password='testpassword123'
        )

    def test_user_creation(self):
        """Test that the user is created with the correct attributes"""
        user = self.user
        self.assertEqual(user.email, 'testuser@example.com')
        self.assertEqual(user.first_name, 'Test')
        self.assertEqual(user.last_name, 'User')
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_staff)
        self.assertFalse(user.is_verified)   # this is false because the user needs to activate their email
        self.assertFalse(user.is_superuser)
        self.assertEqual(user.auth_provider, 'email')

    def test_user_tokens(self):
        """Test that the tokens are generated properly"""
        user = self.user
        tokens = user.tokens()
        self.assertIn('refresh', tokens)
        self.assertIn('access', tokens)


class OneTimePasswordModelTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            email='testotpuser@example.com',
            first_name='OTP',
            last_name='User',
            password='testpassword123'
        )
        self.otp = OneTimePassword.objects.create(
            user=self.user,
            code='123456'
        )

    def test_otp_creation(self):
        """Test that the OTP is created correctly"""
        otp = self.otp
        self.assertEqual(otp.user.email, 'testotpuser@example.com')
        self.assertEqual(otp.user.last_name, 'User')
        self.assertEqual(otp.code, '123456')

    def test_otp_str_method(self):
        """Test the string representation of the OTP"""
        otp = self.otp
        self.assertEqual(str(otp), 'OTP passcode')



from .authentication_views import LoginUserView, TestAuthenticationView, RegisterUserView, VerifyUserEmail, LogoutView
from .password_views import PasswordResetRequestView, PasswordResetConfirm, SetNewPassword



from rest_framework import status
from rest_framework.response import Response
from .serializers import GoogleSignInSerializer, GithubOauthSerializer
from rest_framework.generics import GenericAPIView


# Create your views here.
class GoogleSignInView(GenericAPIView):
    serializer_class = GoogleSignInSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid(raise_exception=True):
            data = ((serializer.validated_data)['access_token'])
            return Response(data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        

class GithubSignInView(GenericAPIView):
    serializer_class = GithubOauthSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid(raise_exception=True):
            data = ((serializer.validated_data)['code'])
            return Response(data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# Generated by Django 5.0.6 on 2024-10-12 18:17

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('email', models.EmailField(max_length=255, unique=True, verbose_name='Email Address')),
                ('first_name', models.CharField(max_length=150, verbose_name='First name')),
                ('last_name', models.CharField(max_length=150, verbose_name='Last name')),
                ('is_staff', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('is_superuser', models.BooleanField(default=False)),
                ('is_verified', models.BooleanField(default=False)),
                ('date_joined', models.DateTimeField(auto_now_add=True)),
                ('last_login', models.DateTimeField(auto_now=True)),
                ('auth_provider', models.CharField(default='email', max_length=50)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OneTimePassword',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=6, unique=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.contrib.auth.models import BaseUserManager
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.utils.translation import gettext_lazy as _


class UserManager(BaseUserManager):
    def email_validator(self, email):
        try:
            validate_email(email)
        except ValidationError:
            raise ValueError(_("Please enter a valid email address"))
        

    def create_user(self, email, first_name, last_name, password, **extra_fields):
        if email:
            email=self.normalize_email(email)
            self.email_validator(email)
        else:
            raise ValueError(_("An Email Address is required"))
        if not first_name:
            raise ValueError(_("First Name is required"))
        if not last_name:
            raise ValueError(_("Last Name is required"))
        
        user = self.model(email=email, first_name=first_name, last_name=last_name, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)

        return user
    
    def create_superuser(self, email, first_name, last_name, password, **extra_fields):
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)
        extra_fields.setdefault("is_verified", True)

        if extra_fields.get("is_staff") is not True:
            raise ValueError (_("is staff must be true for admin user"))
        
        if extra_fields.get("is_superuser") is not True:
            raise ValueError (_("is superuser must be true for admin user"))

        user = self.create_user(
            email, first_name, last_name, password, **extra_fields
        )
        return user


import os
from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    documentation = f.read()

setup(
    name='authease',       # Package name
    version='1.2.0',        # Version of the package
    description='Authease is a lightweight and flexible authentication package for Django, offering essential tools for secure user authentication, including JWT support, with minimal setup required.',
    author='Oluwaseyi Ajayi', 
    author_email='oluwaseyitemitope456@gmail.com',
    url='https://github.com/Oluwatemmy/authease',  # URL to the GitHub repo
    packages=find_packages(),           # Automatically find all packages in the project
    include_package_data=True,          # Include other files like migrations
    install_requires=[
        'Django>=5.0.6',
        'djangorestframework>=3.15.1',
        'python-dotenv>=1.0.1',
        'django-environ>=0.11.2',
        'djangorestframework-simplejwt>=5.3.1',
        'google-api-python-client>=2.136.0',
        'environs>=11.0.0',
        'marshmallow>=3.21.3',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Framework :: Django',
        'Development Status :: 3 - Alpha',
    ],
    long_description=documentation,
    long_description_content_type='text/markdown',
    python_requires='>=3.6',        # Python version compatibility

    project_urls={                   # Optional: additional URLs
        'Documentation': 'https://github.com/Oluwatemmy/authease#readme',
        'Source': 'https://github.com/Oluwatemmy/authease',
        'Issues': 'https://github.com/Oluwatemmy/authease/issues',
    },
)



# Generated by Django 5.1.3 on 2024-11-09 14:12

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_core', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'User', 'verbose_name_plural': 'Users'},
        ),
        migrations.AlterField(
            model_name='onetimepassword',
            name='code',
            field=models.CharField(max_length=6, unique=True, validators=[django.core.validators.RegexValidator(message='Code must be 6 digits.', regex='^\\d{6}$')]),
        ),
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(help_text='Required. Enter a valid email address.', max_length=255, unique=True, verbose_name='Email Address'),
        ),
    ]



from django.test import TestCase
from django.urls import reverse
from authease.auth_core.models import User, OneTimePassword
from rest_framework import status
from django.core import mail
from rest_framework.test import APIClient
from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
from django.utils.encoding import smart_bytes, force_str


class VerifyUserEmailViewTests(TestCase):
    def setUp(self):
        self.verify_email_url = reverse('verify-email')  
        self.user_data = {
            "email": 'testuser@example.com',
            "password": 'testpassword123',
            "first_name": 'Test',
            "last_name": 'User'
        }
        
        # Create a test user
        self.user = User.objects.create_user(**self.user_data)

        self.client = APIClient() 
        
        # Generate a valid OTP code for the user
        self.valid_otp_code = '123456'
        self.otp = OneTimePassword.objects.create(user=self.user, code=self.valid_otp_code)
    
    def test_verify_email_success(self):
        """
        Test successful email verification when the OTP is valid and user is not verified.
        """
        self.user.is_verified = False   # the user has not been verified yet, so it is going to be false until they verify their account
        self.user.save()

        response = self.client.post(self.verify_email_url, {'otp': self.valid_otp_code}, format='json')
        self.user.refresh_from_db()  # Refresh user from DB to get updated is_verified status

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(self.user.is_verified)
        self.assertEqual(response.data['message'], 'Account email verified successfully')

    def test_verify_email_already_verified(self):
        """
        Test email verification when the user is already verified.
        """
        self.user.is_verified = True  # Simulate already verified user
        self.user.save()

        response = self.client.post(self.verify_email_url, {'otp': self.valid_otp_code}, format='json')
        self.user.refresh_from_db()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertTrue(self.user.is_verified)
        self.assertEqual(response.data['message'], 'User is already verified')

    def test_verify_email_invalid_otp(self):
        """
        Test email verification with an invalid OTP code.
        """
        invalid_otp_code = '654321'  # OTP that does not exist in the database

        response = self.client.post(self.verify_email_url, {'otp': invalid_otp_code}, format='json')

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(response.data['message'], 'Passcode does not exist')

    def test_verify_email_missing_otp(self):
        """
        Test email verification when OTP is not provided in the request.
        """
        response = self.client.post(self.verify_email_url, {}, format='json')

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)  # Expecting not found because the code was not found
        self.assertIn('message', response.data)  # Ensure that the 'message' field is mentioned in the error


class PasswordResetRequestViewTests(TestCase):
    def setUp(self):
        self.password_reset_url = reverse('password_reset')
        self.user_data = {
            "email": 'testuser@example.com',
            "password": 'testpassword123',
            "first_name": 'Test',
            "last_name": 'User'
        }

        # Create a test user
        self.user = User.objects.create_user(**self.user_data)
        self.user.is_verified = True
        self.user.save()

        self.client = APIClient()

    def test_password_reset_request_success(self):
        """
        Test successful password reset request with a valid email.
        """
        response = self.client.post(self.password_reset_url, {'email': self.user_data['email']}, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['message'], 'Password reset email sent successfully')

        # Ensure that an email has been sent
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn(self.user_data['email'], mail.outbox[0].to)
        self.assertIn("Reset your Password", mail.outbox[0].subject)

    def test_password_reset_request_invalid_email(self):
        """
        Test password reset request with an invalid email format.
        """
        invalid_email = "invalid-email-format"

        response = self.client.post(self.password_reset_url, {'email': invalid_email}, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)  # Ensure email error is mentioned
        self.assertEqual(response.data['email'][0], 'Enter a valid email address.')

        # Ensure no email is sent
        self.assertEqual(len(mail.outbox), 0)

    def test_password_reset_request_unregistered_email(self):
        """
        Test password reset request with an email not registered in the system.
        """
        unregistered_email = "unregistered@example.com"

        response = self.client.post(self.password_reset_url, {'email': unregistered_email}, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)  # The response will return a bad request error
        self.assertEqual(str(response.data['error'][0]), 'User with this email does not exist')

        # Ensure no email is sent for an unregistered email
        self.assertEqual(len(mail.outbox), 0)

    def test_password_reset_request_missing_email(self):
        """
        Test password reset request with missing email field.
        """
        response = self.client.post(self.password_reset_url, {}, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)  # Ensure the email field is mentioned in the error
        self.assertEqual(response.data['email'][0], 'This field is required.')

    def test_password_reset_request_empty_email(self):
        """
        Test password reset request with an empty email field.
        """
        response = self.client.post(self.password_reset_url, {'email': ''}, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('email', response.data)  # Ensure the email field is mentioned in the error
        self.assertEqual(response.data['email'][0], 'This field may not be blank.')

    def test_password_reset_request_unverified_user(self):
        """
        Test password reset request with an unverified user.
        """
        self.user.is_verified = False
        self.user.save()

        response = self.client.post(self.password_reset_url, {'email': self.user_data['email']}, format='json')

        # Expecting a 400 Bad Request because the user is not verified
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        # Check that the response contains the correct error message
        self.assertEqual(str(response.data['error'][0]), 'Email is not verified. Please verify your email before resetting the password.')

        # Ensure no email is sent for an unverified user
        self.assertEqual(len(mail.outbox), 0)


class PasswordResetConfirmTests(TestCase):
    def setUp(self):
        # Create a user for testing
        self.user = User.objects.create_user(
            email="testuser@example.com",
            password="testpassword123",
            first_name="Test",
            last_name="User",
            is_verified=True  # We assume the user has verified their email
        )
        self.token_generator = PasswordResetTokenGenerator()
        self.valid_uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        self.valid_token = self.token_generator.make_token(self.user)
        self.invalid_token = "invalid-token"
        self.url = reverse('password-reset-confirm', kwargs={'uidb64': self.valid_uidb64, 'token': self.valid_token})

    def test_password_reset_confirm_valid_token(self):
        """
        Test password reset confirmation with a valid token and UID.
        """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["success"], True)
        self.assertEqual(response.data["message"], "Valid token, please reset your password")
        self.assertEqual(response.data["uidb64"], self.valid_uidb64)
        self.assertEqual(response.data["token"], self.valid_token)

    def test_password_reset_confirm_invalid_token(self):
        """
        Test password reset confirmation with an invalid token.
        """
        invalid_token_url = reverse('password-reset-confirm', kwargs={'uidb64': self.valid_uidb64, 'token': self.invalid_token})
        response = self.client.get(invalid_token_url)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["message"], "Invalid token")

    def test_password_reset_confirm_invalid_token_uid(self):
        """
        Test password reset confirm with an invalid token and UID
        """
        invalid_uidb64_url = reverse('password-reset-confirm', kwargs={'uidb64': 'invalid_uidb32', 'token': 'invalid_token'})
        response =self.client.get(invalid_uidb64_url)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["message"], "Invalid token")

    def test_password_reset_confirm_invalid_uid(self):
        """
        Test password reset confirmation with an invalid UID.
        """
        invalid_uidb64 = urlsafe_base64_encode(smart_bytes(99999))  # Non-existent user ID
        invalid_uid_url = reverse('password-reset-confirm', kwargs={'uidb64': invalid_uidb64, 'token': self.valid_token})
        response = self.client.get(invalid_uid_url)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["message"], "Invalid user")

    def test_password_reset_confirm_malformed_uid(self):
        """
        Test password reset confirmation with a malformed UID that raises a DjangoUnicodeDecodeError.
        """
        malformed_uidb64 = "!!invalid-uidb64!!"
        malformed_uid_url = reverse('password-reset-confirm', kwargs={'uidb64': malformed_uidb64, 'token': self.valid_token})
        response = self.client.get(malformed_uid_url)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["message"], "Invalid token")

    def test_password_reset_confirm_token_already_used(self):
        """
        Test password reset confirmation when the token has already been used.
        """
        # Invalidate the token by resetting the user's password, simulating that the token has already been used
        self.user.set_password("newpassword123")
        self.user.save()

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data["message"], "Invalid token")


class SetNewPasswordTests(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            email='testuser@example.com',
            password='oldpassword123',
            first_name='Test',
            last_name='User',
            is_verified=True
        )
        self.uidb64 = urlsafe_base64_encode(smart_bytes(self.user.id))
        self.valid_token = PasswordResetTokenGenerator().make_token(self.user)
        self.set_new_password_url = reverse('set-new-password')

    def test_successful_password_reset(self):
        """
        Test successful password reset with valid token and data.
        """
        data = {
            'uidb64': self.uidb64,
            'token': self.valid_token,
            'password': 'newpassword123',
            'confirm_password': 'newpassword123'
        }

        response = self.client.patch(self.set_new_password_url, data, format='json', content_type='application/json')
        
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['message'], 'Password reset successful')
        
        # Check that the password has been updated
        self.user.refresh_from_db()
        self.assertTrue(self.user.check_password('newpassword123'))

    def test_password_reset_invalid_token(self):
        """
        Test password reset with an invalid token.
        """
        invalid_token = 'invalid-token'
        data = {
            'uidb64': self.uidb64,
            'token': invalid_token,
            'password': 'newpassword123',
            'confirm_password': 'newpassword123'
        }

        response = self.client.patch(self.set_new_password_url, data, format='json', content_type='application/json')
        
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'The reset link is invalid')

    def test_password_reset_invalid_uidb64(self):
        """
        Test password reset with an invalid uidb64.
        """
        invalid_uidb64 = 'invalid-uidb64'
        data = {
            'uidb64': invalid_uidb64,
            'token': self.valid_token,
            'password': 'newpassword123',
            'confirm_password': 'newpassword123'
        }

        response = self.client.patch(self.set_new_password_url, data, format='json', content_type='application/json')

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], 'The reset link is invalid')

    def test_password_reset_mismatched_passwords(self):
        """
        Test password reset with mismatched new_password and confirm_password.
        """
        data = {
            'uidb64': self.uidb64,
            'token': self.valid_token,
            'password': 'newpassword123',
            'confirm_password': 'differentpassword'
        }

        response = self.client.patch(self.set_new_password_url, data, format='json', content_type='application/json')
        
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(response.data['detail'], "Password and Confirm Password doesn't match")

    def test_password_reset_missing_fields(self):
        """
        Test password reset when some required fields are missing (e.g., token, password).
        """
        # Missing password field
        data = {
            'uidb64': self.uidb64,
            'token': self.valid_token,
            'password': '',
            'confirm_password': 'newpassword123'
        }

        response = self.client.patch(self.set_new_password_url, data, format='json', content_type='application/json')
        
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('password', response.data)  # Ensure password field is flagged as missing

